/**
  ******************************************************************************
  * @file    HMC5883L.c
  * @author  Waveshare Team
  * @version V1.0
  * @date    29-August-2014
  * @brief   This file includes the HMC5883L driver functions
  
  ******************************************************************************
  * @attention
  *
  * THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
  * WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
  * TIME. AS A RESULT, WAVESHARE SHALL NOT BE HELD LIABLE FOR ANY
  * DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
  * FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
  * CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
  *
  ******************************************************************************
  */



#include "HMC5883L.h"
#include "I2C.h"

int16_t MagAngle[3];

/**
  * @brief  write an byte to the register adrress of HMC5883L
  * @param  RegAddr: the register adrress of HMC5883L
  * @param Data: the data would be writen to the specified register address
  * @retval None
  */
void HMC5883L_WriteReg(uint8_t RegAddr, uint8_t Data) 
{
	I2C_WriteOneByte(HMC5883L_DEVICE_ADDR, RegAddr, Data);
}

/**
  * @brief  Initializes HMC5883L GPIO
  * @param  None
  * @retval None
  */
void HMC5883L_Init(void)
{
	HMC5883L_WriteReg(HMC5883L_CRA, (HMC5883L_SAMPLE_AVERAGE << 5) | (HMC5883L_DOR << 2) | HMC5883L_MM);
	HMC5883L_WriteReg(HMC5883L_CRB, (HMC5883L_GAIN << 5) & 0xE0);
	HMC5883L_WriteReg(HMC5883L_MR, HMC5883L_CMM);
	Delay_ms(100); //6ms
}

/**
  * @brief  Digital filter
  * @param *pIndex:
  * @param *pAvgBuffer:
  * @param InVal:
  * @param pOutVal:
  *
  * @retval None
  *               
  */
void HMC5883L_CalAvgValue(uint8_t *pIndex, int16_t *pAvgBuffer, int16_t InVal, int16_t *pOutVal)
{	
	uint8_t i;
	
	*(pAvgBuffer + ((*pIndex) ++)) = InVal;
  	*pIndex &= 0x07;
  	
  	*pOutVal = 0;
	for(i = 0; i < 8; i ++) 
  	{
    	*pOutVal += *(pAvgBuffer + i);
  	}
  	*pOutVal >>= 3;
}


/**
  * @brief  Read the datas from HMC5883L data output register
  * @param  *pData: points to a buffer to which will be writen 
  * @retval None
  */
void HMC5883L_DataOutRegister(int16_t *pData)
{
	uint8_t Buffer[6] = {0};
	int16_t AvgBuff[3] = {0};
	static HMC5883L_AvgTypeDef HMC5883L_Filter[3];

	if(NULL == pData)
	{
		return;
	}
	
	I2C_ReadBuff(HMC5883L_DEVICE_ADDR, 0x03, 6, Buffer);

	HMC5883L_CalAvgValue(&HMC5883L_Filter[0].Index, HMC5883L_Filter[0].AvgBuffer, ((int16_t)Buffer[0] << 8) | Buffer[1], (int16_t *)AvgBuff);
	HMC5883L_CalAvgValue(&HMC5883L_Filter[1].Index, HMC5883L_Filter[1].AvgBuffer, ((int16_t)Buffer[4] << 8) | Buffer[5], (int16_t *)AvgBuff + 1);
	HMC5883L_CalAvgValue(&HMC5883L_Filter[2].Index, HMC5883L_Filter[2].AvgBuffer, ((int16_t)Buffer[2] << 8) | Buffer[3], (int16_t *)AvgBuff + 2);
	
	*pData = *AvgBuff;
	*(pData + 1) = *(AvgBuff + 1);
	*(pData + 2) = *(AvgBuff + 2);
}


/**
  * @brief  figures out the angle 
  * @param  MagAngleX: magnetometer X
  * @param  MagAngleY: magnetometer Y
  * @param  *Angle: angle figured out from magnetometer X and magnetometer Y
  * @retval None
  */

void HMC5883L_Compass(int16_t MagAngleX, int16_t MagAngleY, float *Angle)
{
	if(MagAngleX > 0x7fff) MagAngleX -= 0xffff;
    if(MagAngleY > 0x7fff) MagAngleY -= 0xffff;

    *Angle = atan2(MagAngleY, MagAngleX) * (180 / 3.14159265);
}

/******************* (C) COPYRIGHT 2014 Waveshare *****END OF FILE*******************/

